class ToolsLauncher extends Actor config(UltimateTools);

//the central class of the Ultimate Tools mod
//handles installation when run for the first time
//keeps master copies of all variables, and sends them to each client
//welcomes all clients to server and makes sure they are set up properly

//misc vars
struct APlayer
{
	var string UBI;
	var string NickName;
};

//config vars
var config int PreMatchPauseTime;//how long to pause at the beginning of each match
var config bool IncludeWeaponName;//should the name of the gun be included in the death message
var config bool CallHeadshots;//should messages include if it's a headshot
var config bool AllowZulu;//should players be allowed to use zulu
var config bool AutoSwitchOnWin;//should the map switch to the next when a team gets enough wins
var config array<string> MutedPlayer;//UBI ids of muted players
var config string DeathMessage[30];
var config string SuicideMessage[30];
var config string BombMessage[30];
var config string HeadshotMessage[30];
var config string LimitedAdminPass;//if not "", adminlogin with this will give ability to kick, pause, change map, or switch
var config bool AllowMapVote;//allow players to vote on the next available map
var config float AccuracyModifier;//change each player's accuracy
var config array<APlayer> RegisteredPlayers;//list of all players that have registered and protected nicknames
var config bool KickSpectatorsWhenFull;//should the server auto kick a spectator to make room on a full server

//running vars
var int TimePaused;//how long it has been paused so far
var int vNext,vSkip;//how many votes for and against the next map
var string map1,map2;//names of next 2 maps
var int mCur,mNext;//id numbers of current map, next map
var bool bSkipWinning;//is the votes to skip winning?
var bool bStart,bPre;//pause has been started, in a prematch state
var bool bEndedGame;//ended the game due to one team getting majority wins
var array<UTController> UTList;//list of all players that have been welcomed and sent variables

//INSTALLATION
event PreBeginPlay()
{
	local R6Mod Mod;
	local GameEngine GE;
	local int i;
	local bool bFound;
	local class<R6BroadcastHandler> BClass;
	super.PreBeginPlay();
	log("   ----------------------------------",'UTools');
	log("     Ultimate Tools BETA loaded ...",'UTools');
	log("     (c) 2015 by Twi",'UTools');
	log("   ----------------------------------",'UTools');
	//make sure our modded classes get loaded!
	if ( Level.NetMode != NM_Client )
	{
		//R6MOD
		Mod = GetModMgr().m_pCurrentMod;
		if ( InStr(Caps(Mod.m_PlayerCtrlToSpawn),".UTCONTROLLER") == -1 )//controller class
		Mod.m_PlayerCtrlToSpawn = "UltimateTools.UTController";
		if ( InStr(Caps(Mod.m_DefaultLightPawn),".UTPAWNLIGHT") == -1 )//light pawn
		Mod.m_DefaultLightPawn = "UltimateTools.UTPawnLight";
		if ( InStr(Caps(Mod.m_DefaultMediumPawn),".UTPAWNMEDIUM") == -1 )//med pawn
		Mod.m_DefaultMediumPawn = "UltimateTools.UTPawnMedium";
		if ( InStr(Caps(Mod.m_DefaultHeavyPawn),".UTPAWNHEAVY") == -1 )//heavy pawn
		Mod.m_DefaultHeavyPawn = "UltimateTools.UTPawnHeavy";
		if ( InStr(Caps(Mod.m_DefaultPilotPawn),".UTPAWNPILOT") == -1 )//pilot pawn
		Mod.m_DefaultPilotPawn = "UltimateTools.UTPawnPilot";
		Mod.SaveConfig("..//mods//" $ Mod.m_szKeyWord $ ".mod");//save
		//GAME ENGINE
		GE = GameEngine(FindObject("Transient.GameEngine0",class'GameEngine'));
		i = 0;
		while ( i < GE.ServerPackages.length )
		{
			if ( Caps(GE.ServerPackages[i]) == "ULTIMATETOOLS" )
			bFound = true;
			else if ( Caps(GE.ServerPackages[i]) == "SM2" )//uninstall sm2
			{
				GE.ServerPackages.Remove(i,1);
				GE.SaveConfig("..//mods//" $ Mod.m_szKeyWord $ ".mod");//save
			}
			i++;
		}
		if ( !bFound )
		{
			GE.ServerPackages[GE.ServerPackages.length] = "UltimateTools";
			GE.SaveConfig("..//mods//" $ Mod.m_szKeyWord $ ".mod");//save
		}
		i = 0;
		while ( i < GE.ServerActors.length )//uninstall SM2!
		{
			if ( Caps(GE.ServerActors[i]) == "SM2.SM2LAUNCHER" )
			{
				GE.ServerActors.Remove(i,1);
				GE.SaveConfig("..//mods//" $ Mod.m_szKeyWord $ ".mod");//save
			}
			i++;
		}
		SaveConfig();//create INI file
	}
	//broadcasthandler
	//for muting player feature
	class'R6MultiPlayerGameInfo'.default.BroadcastHandlerClass = "UltimateTools.UTBroadcast";
	if ( Level.Game.BroadcastHandler != none )
	{
		BClass = class<R6BroadcastHandler>(DynamicLoadObject("UltimateTools.UTBroadcast",class'class'));
		Level.Game.BroadcastHandler = spawn(BClass);
	}
}

//set up timer
//set up prematch pause if needed
//get the ids of current and next map
event PostNetBeginPlay()
{
	local R6ServerInfo pServerOptions;
	super.PostNetBeginPlay();
	if ( PreMatchPauseTime > 0 )
	{
		bPre = true;//pre game, needs a pause
	}
	pServerOptions = class'Actor'.static.GetServerOptions();
	if ( pServerOptions != none && pServerOptions.m_ServerMapList != none )
	{
		mCur = Level.Game.GetCurrentMapNum();
		mNext = pServerOptions.m_ServerMapList.GetNextMapIndex(-2);
	}
	else
	log("****************  ISSUE: NO SERVER OPTIONS OR MAPLIST");
	SetTimer(1.0,true);
}

//if bPre, sets up the prematch pause time
//initializes vars on ut controller, both clientside and serverside
//handles autoswitching
function Timer()
{
	local Controller C;
	local UTController UT,UP;
	local int i,j,k;
	local bool bFound;
	local R6GameReplicationInfo GRI;
	local R6AdversarialTeamGame ATG;
	local PlayerMenuInfo playerMenuInfo1;
	local R6ServerInfo pServerOptions;
	local array<UTController> US;
	super.Timer();
	GetMapNames();//set the map1 and map2 variables
	C = Level.ControllerList;//for sending all initial vars to PCs
	while ( C != none )
	{
		UT = UTController(C);
		if ( UT != none )
		{
			if ( UT.m_TeamSelection != PTS_UnSelected )//only once you've selected a team!
			{
				i = 0;
				while ( i < UTList.length )//look through list to see if we've already initialized the PC
				{
					if ( UT == UTList[i] )
					bFound = true;
					i++;
				}
				if ( !bFound )//not in the greeted list yet ... send a greeting and send all vars
				{
					UTList[UTList.length] = UT;//add to list so we don't do this more than once
					UT.Initialize(self);//send the PC the launcher - self
					UT.ClientMessage("The Ultimate Server Tools Mod by .Twi loaded ... type 'help' for more info");//welcome
					UT.SetWeaponName(IncludeWeaponName);//should death messages display weapon names
					UT.SetHeadshots(CallHeadshots);//should use headshot messages
					UT.SetZulu(AllowZulu);//should let players use zulu go-code
					UT.SetLimitedAdminPass(LimitedAdminPass);//send the limited admin password
					UT.SetMapNames(map1,map2);//tell the client what the map names are
					UT.SetAimLevel(AccuracyModifier);//tell the client what the modified accuracy values are
					j = 0;
					while ( j < MutedPlayer.length )//MUTE THE PLAYER
					{
						if ( MutedPlayer[j] == UT.PlayerReplicationInfo.m_szUbiUserID )//if UBI is found in the list, mute them
						{
							UT.SetMuted(true);
						}
						j++;
					}
					UT.ReceiveDM(1,DeathMessage);//send messages
					UT.ReceiveDM(2,SuicideMessage);
					UT.ReceiveDM(3,BombMessage);
					UT.ReceiveDM(4,HeadshotMessage);
					CanUseNickName(UT);//check if the name is allowed
				}
			}
		}
		C = C.NextController;
	}
	if ( bPre )//for pre-match pausing
	{
		if ( Level.Game.IsInState('InBetweenRoundMenu') && !bStart )//we are in a pausable state
		{
			bStart = true;//start the pause
			R6MultiPlayerGameInfo(Level.Game).PauseCountDown();//pause game
			return;
		}
		if ( bStart )//pause is started, need to count
		TimePaused++;
		if ( TimePaused >= PreMatchPauseTime )//countdown reached
		{
			bPre = false;//unpause, game started, no more pauses
			R6MultiPlayerGameInfo(Level.Game).UnPauseCountDown();
		}
	}
	if ( AutoSwitchOnWin )//for switching maps when a team gets enough wins
	{
		if ( !bEndedGame )//make sure we haven't done this function yet
		{
			ATG = R6AdversarialTeamGame(Level.Game);
			if ( ATG != none )//is an adversarial game
			{
				GRI = R6GameReplicationInfo(ATG.GameReplicationInfo);
				if ( !ATG.IsLastRoundOfTheMatch() )//if it's not the last round already (or else the game info code will handle all of this)
				{
					if ( Level.IsGameTypeTeamAdversarial(Level.Game.m_szGameTypeFlag) )//team game
					{
						if ( ATG.GetNbRoundWinner() != -1 )
						{
							if ( float(GRI.m_aTeamScore[ATG.GetNbRoundWinner()]) >= ( float(GRI.m_iRoundsPerMatch) / 2 ) )//if the leading team has won at least half the rounds
							{
								if ( ATG.GetNbRoundWinner() == ATG.c_iAlphaTeam )
								{
									ATG.BroadcastGameMsg("","","GreenTeamWonMatch",ATG.m_sndGreenTeamWonMatch,ATG.GetGameMsgLifeTime());
								}
								else if ( ATG.GetNbRoundWinner() == ATG.c_iBravoTeam )
								{
									ATG.BroadcastGameMsg("","","RedTeamWonMatch",ATG.m_sndRedTeamWonMatch,ATG.GetGameMsgLifeTime());
								}
								pServerOptions = class'Actor'.static.GetServerOptions();//get the server options
								if ( pServerOptions != none && pServerOptions.m_ServerMapList != none )//make sure we have a maplist available
								R6GameInfo(Level.Game).SetJumpingMaps(true,pServerOptions.m_ServerMapList.GetNextMapIndex(-2));//tell game info we are changing maps
								R6GameInfo(Level.Game).BaseEndGame();//end the game
								bEndedGame = true;
							}
						}
					}
					else//deathmatch game
					{
						R6GameReplicationInfo(Level.Game.GameReplicationInfo).RefreshMPInfoPlayerStats();//refresh stats so hopefully the following will work!
						GetFPlayerMenuInfo(0,playerMenuInfo1);//should get the leading player's info?
						if ( float(playerMenuInfo1.iRoundsWon) >= ( float(GRI.m_iRoundsPerMatch) / 2 ) )//if the leading player has won at least half the rounds
						{
							ATG.BroadcastGameMsg("",playerMenuInfo1.szPlayerName,"HasWonTheMatch",none,ATG.GetGameMsgLifeTime());
							pServerOptions = class'Actor'.static.GetServerOptions();//get the server options
							if ( pServerOptions != none && pServerOptions.m_ServerMapList != none )//make sure we have a maplist available
							R6GameInfo(Level.Game).SetJumpingMaps(true,pServerOptions.m_ServerMapList.GetNextMapIndex(-2));//tell game info we are changing maps
							R6GameInfo(Level.Game).BaseEndGame();//end the game
							bEndedGame = true;
						}
					}
				}
			}
		}
	}
	if ( KickSpectatorsWhenFull )//if we should kick spectators
	{
		if ( Level.Game.AtCapacity(false) )//server is full?
		{
			C = Level.ControllerList;//get controllers
			while ( C != none )
			{
				UT = UTController(C);
				if ( UT != none )//get players
				{
					if ( UT.m_TeamSelection == PTS_Spectator )//anyone who's a spectator is fair game to kick
					{
						US[US.length] = UT;
					}
				}
				C = C.NextController;
			}
			if ( US.length > 0 )//at least one player is spectating
			{
				UP = US[rand(US.length)];//get a random player from list
				foreach DynamicActors(class'UTController',UT)
				{
					UT.ClientMessage("[Server] Server is full! Kicking a spectator ...");
					UT.ClientAdminKickOff("Server",UP.PlayerReplicationInfo.PlayerName);
				}
				UP.ClientKickedOut();
				UP.SpecialDestroy();
			}
		}
	}
}

//toggles whether to kick a spectator when full
function ChangeAutoKick(UTController U)
{
	local string s;
	KickSpectatorsWhenFull = !KickSpectatorsWhenFull;
	SaveConfig();
	switch ( KickSpectatorsWhenFull )
	{
		case false:
		s = "off";
		break;
		default:
		s = "on";
		break;
	}
	U.ClientMessage("[Server] Automatically kicking a spectator when server full toggled " $ s $ ".");
}

//removes a death message from the list
//saves config as needed
//updates all players with new message list
//called by a ut controller
function RemoveMsg(int type,int num,UTController U)
{
	local UTController C;
	if ( num > 30 )
	{
		U.ClientMessage("[Server] You must specify a message number from 1 to 30.");
		return;
	}
	else
	num--;
	switch ( type )
	{
		case 1://death
		DeathMessage[num] = "";
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(1,DeathMessage);
		}
		break;
		case 2://suicide
		SuicideMessage[num] = "";
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(2,SuicideMessage);
		}
		break;
		case 3://bomb
		BombMessage[num] = "";
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(3,BombMessage);
		}
		break;
		default://headshot
		HeadshotMessage[num] = "";
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(4,HeadshotMessage);
		}
		break;
	}
	SaveConfig();
	U.ClientMessage("[Server] Message " $ num + 1 $ " removed.");
}

//adds a death message to the list
//saves config as needed
//updates all players with new message list
//called by a ut controller
function AddMsg(int type,string msg,UTController U)
{
	local UTController C;
	local int i;
	i = 0;
	switch ( type )
	{
		case 1://death
		while ( DeathMessage[i] != "" )//find first blank
		{
			i++;
		}
		DeathMessage[i] = msg;
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(1,DeathMessage);
		}
		break;
		case 2://suicide
		while ( SuicideMessage[i] != "" )//find first blank
		{
			i++;
		}
		SuicideMessage[i] = msg;
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(2,SuicideMessage);
		}
		break;
		case 3://bomb
		while ( BombMessage[i] != "" )//find first blank
		{
			i++;
		}
		BombMessage[i] = msg;
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(3,BombMessage);
		}
		break;
		default://headshot
		while ( HeadshotMessage[i] != "" )//find first blank
		{
			i++;
		}
		HeadshotMessage[i] = msg;
		foreach DynamicActors(class'UTController',C)
		{
			C.ReceiveDM(4,HeadshotMessage);
		}
		break;
	}
	SaveConfig();
	U.ClientMessage("[Server] Message added.");
}

//MAP VOTING

//toggles whether or not to allow voting for maps
function SetMapVote(UTController U)
{
	AllowMapVote = !AllowMapVote;
	SaveConfig();
	if ( AllowMapVote )//if turned on, tell the server the voting process
	{
		AlertVotes();
	}
	else
	{
		U.ClientMessage("[Server] Voting for next map to play has been disabled.");
	}
}

//each time the level is reset, give the vote count
//alert clients what the options are
simulated function FirstPassReset()
{
	super.FirstPassReset();
	if ( Level.NetMode != NM_Client )
	{
		AlertVotes();
	}
}

//gets the names of next two maps
function GetMapNames()
{
	map1 = GetMapNamePlus(1);
	map2 = GetMapNamePlus(2);
}

//function should get the map name
//cycles through the map list until it hits a non blank name
//does this twice if passed int 2
function string GetMapNamePlus(int i)//1 will get next map, 2 will get map after that
{
	local R6GameReplicationInfo G;
	local int j;
	local string s;
	G = R6GameReplicationInfo(Level.Game.GameReplicationInfo);
	j = mCur;//start the cycle at our current map
	s = "ERROR";//should not happen but in case
	if ( G == none )//no game replication for some reason, give error
	return s;
	while ( i > 0 )
	{
		s = "";
		while ( s == "" )//gets next non blank map
		{
			j++;
			if ( j > 31 )//array is max 32 - this resets the count at beginning
			j = 0;
			s = G.m_mapArray[j];
		}
		i--;
	}
	return s;
}

//sends a message to server of the status of the votes for maps
function AlertVotes()
{
	if ( AllowMapVote )//only alert if enabled
	{
		R6AbstractGameInfo(Level.Game).BroadcastGameMsg("","Vote for next map: " $ map1 $ " (" $ vNext $ " votes) " $ map2 $ " (" $ vSkip $ " votes)","");
		R6AbstractGameInfo(Level.Game).BroadcastGameMsg("","Use the voting buttons or the console command 'VoteMap'","");
	}
}

//takes a string from the client and converts it to a vote for a map
function ConvertVote(UTController UT,string s)
{
	if ( !AllowMapVote )
	{
		UT.ClientMessage("[Server] Voting for the next map has been disabled on this server.");
		return;
	}
	if ( InStr(caps(map1),caps(s)) != -1 )
	UT.Voted(1);
	else if ( InStr(caps(map2),caps(s)) != -1 )
	UT.Voted(2);
	else
	{
		UT.ClientMessage("[Error] Could not find map with similar name to " $ s $ ".");
		UT.ClientMessage("[Error] Vote for either " $ map1 $ " or " $ map2 $ ".");
	}
}

//controller votes, need to add vote
function ClientVote(UTController UT,int i,optional bool bSwitch)
{
	local string s;
	switch ( i )
	{
		case 1:
		vNext++;
		s = map1;
		if ( bSwitch )
		vSkip--;
		break;
		case 2:
		vSkip++;
		s = map2;
		if ( bSwitch )
		vNext--;
		break;
	}
	if ( vSkip > vNext )
	{
		if ( !bSkipWinning )//havent set flag yet, need to
		{
			Level.Game.SetCurrentMapNum(mNext);//sets us one map ahead, so that end game code will think we should go to next map past that
			bSkipWinning = true;
		}
	}
	else//votes to stay are tied or winning
	{
		if ( bSkipWinning )//flag was set, need to reset
		{
			Level.Game.SetCurrentMapNum(mCur);//sets map back to actual current
			bSkipWinning = false;
		}
	}
	UT.ClientMessage("[Server] You have voted for map " $ s $ ".");
	UT.ClientMessage("[Server] Current total: " $ map1 $ " (" $ vNext $ " votes) " $ map2 $ " (" $ vSkip $ " votes)");
}

//end map voting

//receives a new limited password from an admin
//alerts all players of changed status
function SetNewLimitedPassword(optional string s)
{
	local UTController U;
	LimitedAdminPass = s;
	SaveConfig();
	foreach DynamicActors(class'UTController',U)
	{
		if ( LimitedAdminPass != "" )
		{
			U.ClientMessage("[Server] Limited admin password has been changed.");
			U.ClientMessage("[Server] Limited admins should re-log in now.");
		}
		else
		{
			U.ClientMessage("[Server] Use of limited admin powers has been disabled.");
		}
		U.bIsLimitedAdmin = false;
		U.SetLimitedAdminPass(LimitedAdminPass);
	}
}

//receives a new accuracy value from an admin
//alerts all players of changed status
function SetNewAccuracy(float fa)
{
	local UTController U;
	if ( fa == 0 )
	fa = 1.f;
	AccuracyModifier = fa;
	SaveConfig();
	foreach DynamicActors(class'UTController',U)
	{
		if ( fa == 1.f )
		U.ClientMessage("[Server] An admin has set accuracy values to normal.");
		else
		U.ClientMessage("[Server] An admin has set accuracy values to " $ fa $ "x normal.");
		U.ClientMessage("[Server] Accuracy values will be changed starting next round.");
		U.SetAimLevel(AccuracyModifier);
	}
}

//toggles whether or not to switch when a map is won decisively
function ChangeAutoSwitch(UTController UT)
{
	local string s;
	AutoSwitchOnWin = !AutoSwitchOnWin;
	SaveConfig();
	switch ( AutoSwitchOnWin )
	{
		case false:
		s = "off";
		break;
		default:
		s = "on";
		break;
	}
	UT.ClientMessage("[Server] Switching maps automatically when a team or player gets enough wins toggled " $ s $ ".");
}

//mutes or unmutes a player
//saves config as needed
//alerts muted player
//called from utclass
function ChangeMute(UTController Admin,UTController B)
{
	local int i,j;
	local UTController C;
	local bool bFound;
	//discover if player is muted already
	i = 0;
	while ( i < MutedPlayer.length )
	{
		if ( MutedPlayer[i] == B.PlayerReplicationInfo.m_szUbiUserID )
		{
			bFound = true;
			j = i;
		}
		i++;
	}
	if ( !bFound )//need to mute player
	{
		MutedPlayer[MutedPlayer.length] = B.PlayerReplicationInfo.m_szUbiUserID;//add the id
		B.SetMuted(true);
		foreach DynamicActors(class'UTController',C)//notify
		{
			C.ClientMessage("[Admin] " $ Admin.PlayerReplicationInfo.PlayerName $ " mutes " $ B.PlayerReplicationInfo.PlayerName $ ".");
		}
	}
	else//need to unmute player
	{
		MutedPlayer.Remove(j,1);//remove the id
		B.SetMuted(false);
		foreach DynamicActors(class'UTController',C)//notify
		{
			C.ClientMessage("[Admin] " $ Admin.PlayerReplicationInfo.PlayerName $ " unmutes " $ B.PlayerReplicationInfo.PlayerName $ ".");
		}
	}
	SaveConfig();
}

//toggles weapon name display on or off
//saves to config
//called from utclass
function ChangeWeaponName(UTController UT)
{
	local UTController C;
	local string s;
	IncludeWeaponName = !IncludeWeaponName;
	SaveConfig();
	foreach AllActors(class'UTController',C)
	{
		if ( C != none )
		C.SetWeaponName(IncludeWeaponName);//send to each player
	}
	switch ( IncludeWeaponName )
	{
		case false:
		s = "off";
		break;
		default:
		s = "on";
		break;
	}
	UT.ClientMessage("[Server] Use of weapon names in death messages toggled " $ s $ ".");
}

//toggles headshot message use on or off
//saves config
//called from utclass
function ChangeHeadshots(UTController UT)
{
	local UTController C;
	local string s;
	CallHeadshots = !CallHeadshots;
	SaveConfig();
	foreach AllActors(class'UTController',C)//change the value for all players
	{
		if ( C != none )
		C.SetHeadshots(CallHeadshots);
	}
	switch ( CallHeadshots )
	{
		case false:
		s = "off";
		break;
		default:
		s = "on";
		break;
	}
	UT.ClientMessage("[Server] Use of headshot messages toggled " $ s $ ".");
}

//toggles whether zulu commands can be used
function ToggleZulu(UTController UT)
{
	local UTController C;
	local string s;
	AllowZulu = !AllowZulu;
	SaveConfig();
	foreach AllActors(class'UTController',C)//change for all players
	{
		if ( C != none )
		C.SetZulu(AllowZulu);
	}
	switch ( AllowZulu )
	{
		case false:
		s = "off";
		break;
		default:
		s = "on";
		break;
	}
	UT.ClientMessage("[Server] Use of Zulu go-codes toggled " $ s $ ".");
}

//returns true if nickname not found and can be allowed
//if wantedname is set that means we are testing a name sent
//otherwise tests the player's  name
//seems to work, needs more testing
function bool CanUseNickName(UTController B,optional string WantedName)
{
	local int i,j;
	local string n,c,t;
	local bool bFound;
	bFound = false;
	if ( WantedName != "" )
	t = WantedName;
	else
	t = B.PlayerReplicationInfo.PlayerName;
	i = 0;
	while ( i < RegisteredPlayers.length )
	{
		if ( InStr(caps(t),caps(RegisteredPlayers[i].NickName)) != -1 )//if the nickname was found in the name
		{
			if ( caps(B.PlayerReplicationInfo.m_szUbiUserID) != caps(RegisteredPlayers[i].UBI) )//not the same UBI id
			{
				bFound = true;
				B.ClientMessage("[Server] Part or all of name is registered on this server to another player.");
				B.ClientMessage("[Server] Only that player is permitted to use the phrase '" $ RegisteredPlayers[i].NickName $ "' in their name.");
				if ( WantedName == "" )//if the player name needs to be modified - if it's just a request, no need for this
				{
					B.ClientMessage("[Server] Your name will be changed to conform to server rules.");
					//replace offending string
					j = 0;
					n = "";
					while ( j < len(RegisteredPlayers[i].NickName) )
					{
						n = n $ chr(rand(24) + 65);//build random replacement
						j++;
					}
					c = caps(t);
					ReplaceText(c,caps(RegisteredPlayers[i].NickName),n);
					B.ClientChangeName(c);//set player name to string c
				}
			}
		}
		i++;
	}
	return !bFound;
}

//adds a name to the list of registered names
function RegisterName(R6PlayerController B,string sName,UTController Ask)
{
	local UTController A;
	if ( CanUseNickName(UTController(B),sName) )//check if already registered
	{
		RegisteredPlayers.length = RegisteredPlayers.length + 1;
		RegisteredPlayers[RegisteredPlayers.length-1].NickName = sName;
		RegisteredPlayers[RegisteredPlayers.length-1].UBI = B.PlayerReplicationInfo.m_szUbiUserID;
		SaveConfig();
		B.ClientMessage("[Server] You have been granted exclusive rights to use the name '" $ sName $ "' on this server.");
		Ask.ClientMessage("[Server] Player with UBI " $ B.PlayerReplicationInfo.m_szUbiUserID $ " has been registered with name '" $ sName $ "'.");
		foreach DynamicActors(class'UTController',A)
		{
			CanUseNickName(A);//make sure none of the players are using the newly registered nickname
		}
	}
}

//finds and removes a nickname from registered
function UnRegisterName(string s,UTController Ask)
{
	local int i;
	local bool bFound;
	bFound = false;
	i = 0;
	while ( i < RegisteredPlayers.length )
	{
		if ( InStr(caps(RegisteredPlayers[i].NickName),caps(s)) != -1 )//nickname was found
		{
			Ask.ClientMessage("[Server] Nickname '" $ RegisteredPlayers[i].NickName $ "' for player ID " $ RegisteredPlayers[i].UBI $ " removed from list of registered names.");
			RegisteredPlayers.Remove(i,1);//remove the entry
			SaveConfig();
			bFound = true;
		}
		else
		i++;
	}
	if ( !bFound )
	Ask.ClientMessage("[Error] Nickname '" $ s $ "' not found in list of registered names.");
}

defaultproperties
{
	//MISC FEATURES:
	//accuracy values set normal by default
	AccuracyModifier=1.0
	//prematch pausing set to 35 seconds by default
	PreMatchPauseTime=35
	//zulu go codes allowed by default
	AllowZulu=true
	//auto map switching when a team wins disabled by default
	AutoSwitchOnWin=false
	//voting for the next map to play enabled by default
	AllowMapVote=true
	//limited admin disabled by default
	LimitedAdminPass=""
	//kick spectators from full server disabled by default
	KickSpectatorsWhenFull=false
	//DEATH MESSAGES:
	//weapon name in death messages enabled by default
	IncludeWeaponName=true
	//headshot death messages enabled by default
	CallHeadshots=true
	//list of default death messages
	DeathMessage(0)="% tore & apart"
	DeathMessage(1)="% taught & a lesson"
	SuicideMessage(0)="& decided to end it all"
	SuicideMessage(1)="& took the coward's way out"
	BombMessage(0)="& was blown to pieces by a bomb"
	BombMessage(1)="A bomb went off and incinerated &"
	HeadshotMessage(0)="% blew &'s head off"
	HeadshotMessage(1)="% sent a bullet through &'s head"
	//OTHER:
	//other housekeeping settings
	bHidden=true
	bAlwaysRelevant=true
	bAlwaysTick=true
	RemoteRole=ROLE_SimulatedProxy
}