class UTController extends R6PlayerController config(User);

//the main class for the Ultimate Server Tools Mod
//compile in 16h

//to install:
// - Stop your server, and open your Ravenshield.mod file
// - Add the following line to the bottom of your Ravenshield.mod file: ServerActors=UltimateTools.ToolsLauncher
// - Start your server, and wait 10 seconds to allow the mod to automatically install needed components
// - Stop the server again, and restart it

//features:

//new features:
// - easy installation - automatically sets most of the needed server config upon first running
// - settings and player windows allow easy access to most mod features
// - built-in help tips
// - mute name, say, teamsay, chat, serversay
// - pause console command
// - auto-pause at beginning of match (on and set to 35 by default)
// - maps can auto-change once a team or player gets at least half of the total wins (off by default)
// - limited admin password - allows giving players limited kicking/switching/pausing/mapchanging powers - need to log in each match to use (off by default)
// - players can vote for the map to play after the current one, can be toggled on or off (on by default)
// - admins can modify the accuracy of weapons - fully compatible with Sentinel (v1.7+)
// - server admins can register player nicknames so only that player can use that name
// - server can auto-kick a spectator if the server is full
// - easter egg bonus games with the 'distract' command

//sm2 and improved features:
// - private chat
// - gimp troublesome players
// - kill troublesome players
// - switch player teams - simplified and bug-fixed
// - expanded playerlist for admins
// - start round instantly
// - server messages to all
// - fully customizable death messages, including headshots and weapon names - weapon name and headshot messages can be toggled on or off (on by default)
// - zulu use can be toggled on or off (on by default)

//List of classes:
// - ToolsLauncher		- handles installation, loading the mod, and saving of all variables
// - UT2048				- 2048 easter egg game
// - UT4App				- tic tac toe easter egg game
// - UTApp				- the basic window that apps go inside
// - UTBroadcast		- broadcast handler to prevent mute players from chatting
// - UTController		- handles most of the features, including all console commands
// - UTNavBar			- adds map voting buttons to the between round nav bar
// - UTPawn				- used to figure out method of death, etc, for death messages
// - UTPawnHeavy		- heavy pawn class extending UTPawn
// - UTPawnLight		- light pawn class extending UTPawn
// - UTPawnMedium		- medium pawn class extending UTPawn
// - UTPawnPilot		- pilot pawn class extending UTPawn
// - UTPlatformApp		- platformer easter egg game - hidden ... movement code needs work
// - UTPlayerButton		- a button that has an additional variable to hold a playerreplicationinfo
// - UTPlayersApp		- app that allows easy moderation of players in a server
// - UTSettingsApp		- app that allows quick changes to server settings

//to do:

// - - TESTING AND FIXES:
// - console command issue? appeared to be an issue with pause command when a limited admin, pause has been fixed!

// - - ADMIN ENHANCEMENTS:
// - chat logging? can log all chat into a statlog file, do it through the broadcast handler? would need to send private chat to BH class as well
// - save into a log file or ini file any name a player uses. save it to their ubi

// - - BALANCE AND GAME:
// - detect and punish camping
// - spawn changing/randomizing?
// - respawning?
// - team killer protection? - wat is this
// - autobalancer that takes skill into account?
// - patrol tangos
// - arena deathmatch mode - players all start with pistol, can pick up primary weapons in map
// - solitare?
// - tagging with armpatch text? Pawn.m_ArmPatchTexture
// - modular city map with actors to represent possible locations of buildings, trees, fences, etc

// - - DEATH MESSAGES:
// - death message taunts several kills in a row, killstreaks?

//-----------------------------------------------
//to receive from launcher

var string DeathMessage[30];//repl
var string SuicideMessage[30];//repl
var string BombMessage[30];//repl
var string HeadshotMessage[30];//repl
var bool CallHeadshots;//should messages include if it's a headshot
var bool IncludeWeaponName;//should include weapon in death msg
var bool bZuluAllowed;//allowed to zulu
var string LimitedAdminPass;//passwd for limited admin - can kick, switch, pause, or change maps
var ToolsLauncher Launcher;//the launcher class
var bool bIAmMuted;//server
var string map1,map2;//the names of next two potential maps
var float fAim;//factor to modify aim by - for run n gun
var float CurrentfAim;//current accuracy on client - updated each round - for anticheat compatibility

//-----------------------------------------------
//end receive

//-----------------------------------------------
//general

var UTApp SettingsApp,PlayersApp,XOApp,ViewApp,TwoApp,ScanApp;//PlatApp;//app windows
var array<string> helpfiles;//client
var bool bRequestedUnMute;//server
var bool bIsLimitedAdmin;//can kick, switch, pause, or change maps
var int iVoted;//the map we voted for

//-----------------------------------------------
//end general

//-----------------------------------------------
//file browser

//on admin
var array<string> files,folders;
var string assembledlist;

//on server
var string cdir;//directory to look at
var UTController vtarget;//viewtarget

//on client
var string creplstring;//client-side string that needs to be replicated
var int askerPC;//admin to send data to

//-----------------------------------------------
//end file browser

replication
{
	reliable if ( Role == ROLE_Authority )//vars server sends to client
	DeathMessage,SuicideMessage,BombMessage,HeadshotMessage;
	reliable if ( Role == ROLE_Authority )//server calls to client
	xClientDeathMessage,SetWeaponName,SetHeadshots,ShowSettingsApp,ShowPlayersApp,
	SetMapNames,SetAimLevel,AddConsoleMsg;
	reliable if ( Role < ROLE_Authority )//client calls to server
	ClientWantsChat,ClientWantsGimp,ClientWantsKill,ClientWantsStart,
	ClientWantsBigMsg,ClientWantsSwitch,ClientWantsPause,ClientWantsSetAutoPause,ClientWantsMessage,ClientWantsWeaponName,
	ClientWantsMute,ClientWantsUnMute,ClientWantsToggleHeadshotMsg,ClientWantsToggleZulu,ClientWantsToggleAutoSwitch,
	ClientWantsSettingsApp,ClientWantsPlayersApp,ClientWantsAppLoad,ClientWantsNewLimitedPassword,
	ClientWantsToggleMapVote,ClientWantsMapVote,ClientWantsSetAccuracy,ClientWantsRegister,
	ClientWantsRegisteredList,ClientWantsUnRegister,ClientWantsKickSpec;
}

//---------------------------------------------------------------------------
//								GENERAL
//---------------------------------------------------------------------------

//lists basic commands
//if typed "help all", lists all features
//can search for help on a particular topic
exec function Help(optional string s)
{
	local color c;
	local int i;
	c.R = 0;
	c.B = 255;
	c.G = 255;
	AddMessageToConsole("----------------------- The Ultimate Server Tools Mod BETA -----------------------",c);
	if ( caps(s) == "MORE" )//list all features
	{
		AddMessageToConsole("Even more console commands (admin only):",c);
		AddMessageToConsole("   - ServerSay <msg> - sends a server message to everyone",c);
		AddMessageToConsole("   - Pause - pauses or unpauses the pre-round countdown",c);
		AddMessageToConsole("   - Start - starts the round instantly",c);
		AddMessageToConsole("   - Switch <#> <optional team> - switch a player to another team",c);
		AddMessageToConsole("   - Gimp <#> - gimps a player as punishment",c);
		AddMessageToConsole("   - Kill <#> - kills a player as punishment",c);
		AddMessageToConsole("   - Mute <#> - mutes or unmutes a player",c);
		AddMessageToConsole("   - SetAutoPause <#> - sets how long the server auto-pauses at the beginning of match",c);
		AddMessageToConsole("   - PlayerList - this command now also lists UBI IDs and admin status",c);
		AddMessageToConsole("   - ToggleZulu - allows or disallows the use of zulu go-codes on the server",c);
		AddMessageToConsole("   - ToggleAutoSwitch - turns on or off the automatic map switching when a player or team wins the majority of rounds",c);
		AddMessageToConsole("   - NewLimitedPassword <password> - sets a limited password that gives ability to kick, switch, pause, or change map",c);
		AddMessageToConsole("   - ToggleMapVote - turns on or off the voting to determine the next map to play",c);
		AddMessageToConsole("   - ToggleKickSpectators - turns on or off automatic kicking of a spectator if server is full",c);
		AddMessageToConsole("   - SetAccuracy <#> - modifies the accuracy of all weapons on the server",c);
		AddMessageToConsole("   - Register <#> <name> - registers a specific nickname to be used only by player with ID#",c);
		AddMessageToConsole("   - UnRegister <name> - removes a registered name so anyone can use it",c);
		AddMessageToConsole("   - ToggleHeadshotMsg - turns the use of headshot death messages on or off",c);
		AddMessageToConsole("   - ToggleWeaponName - turns the use of weapon names in death messages on or off",c);
		AddMessageToConsole("(For the next commands, append with 'ADD <msg>' to add a message, or 'REMOVE <#>' to remove a message)",c);
		AddMessageToConsole("   - DeathMsg <optional commands> - views or changes death messages",c);
		AddMessageToConsole("   - SuicideMsg <optional commands> - views or changes suicide messages",c);
		AddMessageToConsole("   - BombMsg <optional commands> - views or changes bomb death messages",c);
		AddMessageToConsole("   - HeadshotMsg <optional commands> - views or changes headshot messages",c);
		AddMessageToConsole(" ",c);
	}
	else if ( s != "" )//searching
	{
		if ( len(s) < 4 )
		{
			AddMessageToConsole("Please enter a word or phrase at least 4 letters long to search for.",c);
			AddMessageToConsole(" ",c);
			return;
		}
		AddMessageToConsole("Searching for help for " $ s $ "...",c);
		AddMessageToConsole(" ",c);
		AddMessageToConsole("Results:",c);
		i = 0;
		while ( i < helpfiles.length )
		{
			if ( InStr(Caps(helpfiles[i]),Caps(s)) != -1 )
			AddMessageToConsole(helpfiles[i],c);
			i++;
		}
		AddMessageToConsole(" ",c);
	}
	else//basic help
	{
		AddMessageToConsole("Created by .Twi - This mod adds new console commands, admin control, and game features to your server",c);
		AddMessageToConsole(" ",c);
		AddMessageToConsole("New console commands for all players:",c);
		AddMessageToConsole("   - Help More - list even more commands and features",c);
		AddMessageToConsole("   - Help <phrase to search for> - get help with Ultimate Server Tools Mod",c);
		AddMessageToConsole("   - Chat <#> - private chat with player",c);
		AddMessageToConsole("   - VoteMap <map name> - vote for the map you want to play next",c);
		AddMessageToConsole("   - AskUnMute - asks an admin to unmute you",c);
		AddMessageToConsole("   - ListNames - view the list of nicknames that are registered on this server",c);
		AddMessageToConsole("New admin features:",c);
		AddMessageToConsole("   - Settings - opens the Settings App window",c);
		AddMessageToConsole("   - Players - opens the Players App window",c);
		AddMessageToConsole(" ",c);
		AddMessageToConsole("Random help tip of the day:",c);
		AddMessageToConsole(helpfiles[rand(helpfiles.length)],c);
		AddMessageToConsole(" ",c);
	}
}

//to do: make a custom app loader?
exec function AppLoad(string s)
{
	ClientWantsAppLoad(s);
}

//receives class name to load from client
//attempts to load the window on server
//BAD IDEA! why load uwindow on server?
//need to send back to client
function ClientWantsAppLoad(coerce string s)
{
//	local class<UWindowDialogClientWindow> App;
//	App = class<UWindowDialogClientWindow>(DynamicLoadObject(s,class'Class'));
//	if ( App.default.Text
	ClientMessage("[Server] App loader not enabled. App " $ s $ " cannot be loaded.");
}

//tic tac toe
exec function XandO()
{
	local R6Console C;
	if ( Level.NetMode != NM_DedicatedServer )//only on client
	{
		C = R6Console(Player.Console);
		if ( C.Root == none )
		return;
		XOApp = UTApp(C.Root.CreateWindow(class'UTApp',1,1,155,90));
		XOApp.MakeAppWindow("Xs & Os",class'UT4App');
	}
}

exec function Distract()
{
	ClientMessage("Try typing 'xando' or 'tzfe' into the console.");
}

exec function TZFE()
{
	local R6Console C;
	if ( Level.NetMode != NM_DedicatedServer )//only on client
	{
		C = R6Console(Player.Console);
		if ( C.Root == none )
		return;
		TwoApp = UTApp(C.Root.CreateWindow(class'UTApp',1,1,260,300));
		TwoApp.MakeAppWindow("2048",class'UT2048');
	}
}

//requests the settings box for the mod
exec function Settings()
{
	ClientWantsSettingsApp();
}

//checks if client can have settings app
function ClientWantsSettingsApp()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	ShowSettingsApp();
}

//displays settings app on client
simulated function ShowSettingsApp()
{
	local R6Console C;
	if ( Level.NetMode != NM_DedicatedServer )//only on client
	{
		C = R6Console(Player.Console);
		if ( C.Root == none )
		return;
		SettingsApp = UTApp(C.Root.CreateWindow(class'UTApp',1,1,190,314));
		SettingsApp.MakeAppWindow("Mod Settings App",class'UTSettingsApp');
	}
}

//requests the players box for the mod
exec function Players()
{
	ClientWantsPlayersApp();
}

//checks if client can have players app
function ClientWantsPlayersApp()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	ShowPlayersApp();
}

//displays the player app
simulated function ShowPlayersApp()
{
	local R6Console C;
	if ( Level.NetMode != NM_DedicatedServer )//only on client
	{
		C = R6Console(Player.Console);
		if ( C.Root == none )
		return;
		PlayersApp = UTApp(C.Root.CreateWindow(class'UTApp',1,1,190,330));
		PlayersApp.MakeAppWindow("Players App",class'UTPlayersApp');
	}
}

//---------------------------------------------------------------------------
//								END GENERAL
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//								RECEIVE FROM LAUNCHER
//---------------------------------------------------------------------------

//gets the launcher reference
function Initialize(ToolsLauncher TL)
{
	Launcher = TL;
}

//gets modified accuracy value
simulated function SetAimLevel(float f)
{
	if ( f == 0 )
	f = 1.f;
	fAim = f;
}

//receives an array of messages from the launcher - serverside function that replicates vars
function ReceiveDM(int type,string msg[30])
{
	local int i;
	i = 0;
	switch ( type )
	{
		case 1:
		while ( i < 30 )
		{
			DeathMessage[i] = msg[i];
			i++;
		}
		break;
		case 2:
		while ( i < 30 )
		{
			SuicideMessage[i] = msg[i];
			i++;
		}
		break;
		case 3:
		while ( i < 30 )
		{
			BombMessage[i] = msg[i];
			i++;
		}
		break;
		default:
		while ( i < 30 )
		{
			HeadshotMessage[i] = msg[i];
			i++;
		}
		break;
	}
}

//tells the client what the two potential map names are
//when it has map names, creates the new nav bar for the vote buttons
//WORKS!
simulated function SetMapNames(string s1,string s2)
{
	local R6Console C;
	local R6MenuInGameMultiPlayerRootWindow MPRoot;
	local R6MenuMPInterWidget Inter;
	map1 = s1;
	map2 = s2;
	if ( Level.NetMode != NM_DedicatedServer )//only on client
	{
		C = R6Console(Player.Console);
		if ( C.Root != none )
		{
			MPRoot = R6MenuInGameMultiPlayerRootWindow(C.Root);
			if ( MPRoot != none )
			{
				Inter = MPRoot.m_pIntermissionMenuWidget;
				if ( Inter != none )
				{
					if ( Inter.m_pInGameNavBar != none )
					{
						Inter.m_pInGameNavBar.HideWindow();
						Inter.m_pInGameNavBar = UTNavBar(Inter.CreateWindow(class'UTNavBar',MPRoot.m_RInterWidget.X,0,MPRoot.m_RInterWidget.W,Inter.m_pMPInterHeader.WinHeight));
						UTNavBar(Inter.m_pInGameNavBar).MapNames(map1,map2);
						Inter.SetInterWidgetMenu(MPRoot.m_szCurrentGameType,MPRoot.m_bActiveBar);//fix bug with weird positioning
					}
				}
			}
		}
	}
}

//receives can we use headshot messages
simulated function SetHeadshots(bool b)
{
	CallHeadshots = b;
}

//receives include weapon name in messages
simulated function SetWeaponName(bool b)
{
	IncludeWeaponName = b;
}

//serverside
//mutes or unmutes a player and alerts them
function SetMuted(bool b)
{
	if ( b )
	{
		bIAmMuted = true;
		ClientMessage("[Server] You have been muted on this server, and are no longer allowed to chat or change names.");
		ClientMessage("[Server] You can use the console command 'AskUnMute' to ask an admin to unmute you.");
	}
	else
	{
		bIAmMuted = false;
		ClientMessage("[Server] You have been unmuted on this server.");
		ClientMessage("[Server] You are now permitted to chat normally.");
	}
}

//serverside
//can zulu go codes be used
function SetZulu(bool bAllowed)
{
	bZuluAllowed = bAllowed;
}

//receives the limited admin password from launcher
function SetLimitedAdminPass(string s)
{
	LimitedAdminPass = s;
}

//---------------------------------------------------------------------------
//								END RECEIVE
//---------------------------------------------------------------------------

exec function ToggleAutoKickSpectators()
{
	ClientWantsKickSpec();
}

exec function ToggleKickSpectators()
{
	ClientWantsKickSpec();
}

exec function ToggleKickSpectator()
{
	ClientWantsKickSpec();
}

function ClientWantsKickSpec()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	Launcher.ChangeAutoKick(self);
}

//MUTE AND ZULU:

exec function Mute(string s)
{
	ClientWantsMute(s);
}

//does the same as mute
exec function UnMute(string s)
{
	Mute(s);
}

//server side
//finds target player and sends reference to launcher
function ClientWantsMute(coerce string s)
{
	local int i,j;
	local UTController B,C;
	local bool bFound;
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( s == "" )
	{
		ClientMessage("[Error] No player chosen. Use 'mute <ID#>' to choose a player to mute or unmute.");
		return;
	}
	B = UTController(FindPlayer(s,true));
	if ( B != none )
	{
		if ( Launcher != none )
		Launcher.ChangeMute(self,B);
	}
	else
	ClientMessage("[Error] Player #" $ s $ " not found.");
}

exec function AskMute()
{
	ClientWantsUnMute();
}

exec function AskUnMute()
{
	ClientWantsUnMute();
}

exec function RequestUnMute()
{
	ClientWantsUnMute();
}

//server function asks to be unmuted
function ClientWantsUnMute()
{
	local UTController UT;
	local bool bFound;
	if ( !bIAmMuted )
	{
		ClientMessage("[Error] You are not currently muted on this server.");
		return;
	}
	if ( bRequestedUnMute )
	{
		ClientMessage("[Error] You have already requested an unmute this match.");
		ClientMessage("[Error] Please request an unmute later.");
		return;
	}
	foreach DynamicActors(class'UTController',UT)
	{
		if ( UT.CheckAuthority(1) )
		{
			UT.ClientMessage("[Server] " $ PlayerReplicationInfo.PlayerName $ " has requested to be unmuted.");
			UT.ClientMessage("[Server] To do so, use the 'mute <ID#>' command.");
			bFound = true;
		}
	}
	if ( bFound )
	{
		bRequestedUnMute = true;
		ClientMessage("[Server] Admins have been notified of your unmute request.");
	}
	else
	{
		ClientMessage("[Error] There is currently no admin player on the server.");
		ClientMessage("[Error] Please request an unmute later.");
	}
}

//mute naming
//also checks if name is not already registered
simulated function ServerChangeName(string s)
{
	if ( bIAmMuted )//muted player
	{
		SetMuted(true);
		return;
	}
	else if ( ( Launcher != none ) && ( Launcher.CanUseNickName(self,s) ) )//if we are allowed to use this name
	super.ServerChangeName(s);
}

//end zulu spam, mute go code
function ServerSendGoCode(EGoCode eGo)
{
	if ( eGo == GOCODE_Zulu )
	{
		if ( !bZuluAllowed )//zulu disabled
		return;
	}
	if ( bIAmMuted )//muted player
	{
		SetMuted(true);
		return;
	}
	super.ServerSendGoCode(eGo);
}

//END MUTE

//REGISTER NAMES

//lists all registered names
exec function ListNames()
{
	Register(0);
}

exec function NameList()
{
	Register(0);
}

exec function NamesList()
{
	Register(0);
}

exec function ListRegisteredNames()
{
	Register(0);
}

//can register a nickname to a player
//or displays all registered names
exec function Register(int i,optional string s)
{
	if ( ( i != 0 ) && ( s != "" ) )
	ClientWantsRegister(i,s);
	else
	ClientWantsRegisteredList();
}

//removes a nickname from the list
exec function UnRegister(string s)
{
	ClientWantsUnRegister(s);
}

//remove nickname
function ClientWantsUnRegister(string s)
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	Launcher.UnRegisterName(s,self);
}

//registers a nickname
function ClientWantsRegister(int i,string s)
{
	local R6PlayerController B;
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	B = R6PlayerController(FindPlayer(string(i),true));
	if ( B == none )
	{
		ClientMessage("[Error] Player #" $ i $ " not found.");
		return;
	}
	if ( Launcher != none )
	Launcher.RegisterName(B,s,self);
}

//displays registered nicknames
function ClientWantsRegisteredList()
{
	local int i;
	if ( Launcher == none )
	return;
	AddConsoleMsg("----  List of registered nicknames on this server ----");
	i = 0;
	while ( i < Launcher.RegisteredPlayers.length )
	{
		AddConsoleMsg(Launcher.RegisteredPlayers[i].NickName);
		i++;
	}
}

simulated function AddConsoleMsg(string s)
{
	local color c;
	c.R = 0;
	c.B = 255;
	c.G = 255;
	AddMessageToConsole(s,c);
}

//END REGISTER

//RUN N GUN

exec function SetAccuracy(optional float fa)
{
	if ( fa == 0 )
	{
		ClientMessage("[Server] Type 'SetAccuracy' followed by a number to modify all weapons' accuracy.");
		ClientMessage("[Server] Default R6 accuracy is 1.0, default Run N Gun mod accuracy is 0.5.");
		ClientMessage("[Server] Numbers smaller than 1 are more accurate, larger than 1 are less accurate.");
		return;
	}
	ClientWantsSetAccuracy(fa);
}

function ClientWantsSetAccuracy(float fa)
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	Launcher.SetNewAccuracy(fa);
}

//modified from Run N Gun mod
//allows changing accuracy
function Adjust()
{
	if ( bOnlySpectator )
	{
		return;
	}
	CurrentfAim = fAim;//update each round, for anticheat
	if ( Pawn.m_WeaponsCarried[0] != None )
	{
		R6Weapons(Pawn.m_WeaponsCarried[0]).m_stAccuracyValues.fWalkingFastAccuracy = R6Weapons(Pawn.m_WeaponsCarried[0]).default.m_stAccuracyValues.fWalkingFastAccuracy * fAim;
		R6Weapons(Pawn.m_WeaponsCarried[0]).m_stAccuracyValues.fRunningAccuracy = R6Weapons(Pawn.m_WeaponsCarried[0]).default.m_stAccuracyValues.fWalkingFastAccuracy * fAim;
		R6Weapons(Pawn.m_WeaponsCarried[0]).m_stAccuracyValues.fWalkingAccuracy = FMax(R6Weapons(Pawn.m_WeaponsCarried[0]).default.m_stAccuracyValues.fWalkingAccuracy * fAim,R6Weapons(Pawn.m_WeaponsCarried[0]).default.m_stAccuracyValues.fBaseAccuracy);
	}
	if ( Pawn.m_WeaponsCarried[1] != None )
	{
		R6Weapons(Pawn.m_WeaponsCarried[1]).m_stAccuracyValues.fWalkingFastAccuracy = R6Weapons(Pawn.m_WeaponsCarried[1]).default.m_stAccuracyValues.fWalkingFastAccuracy * fAim;
		R6Weapons(Pawn.m_WeaponsCarried[1]).m_stAccuracyValues.fRunningAccuracy = R6Weapons(Pawn.m_WeaponsCarried[1]).default.m_stAccuracyValues.fWalkingFastAccuracy * fAim;
		R6Weapons(Pawn.m_WeaponsCarried[1]).m_stAccuracyValues.fWalkingAccuracy = FMax(R6Weapons(Pawn.m_WeaponsCarried[1]).default.m_stAccuracyValues.fWalkingAccuracy * fAim,R6Weapons(Pawn.m_WeaponsCarried[1]).default.m_stAccuracyValues.fBaseAccuracy);
	}
}

function PossessInit(Pawn P)
{
	super.PossessInit(P);
	if ( Level.Game.m_bGameStarted )
	{
		Adjust();
	}
}

function CountDownPopUpBoxDone()
{
	super.CountDownPopUpBoxDone();
	Adjust();
}

//END RUN N GUN

//MAP VOTING

exec function MapVote(optional string s)
{
	VoteMap(s);
}

//if no map specified, tell them how to vote
//otherwise send vote to server
exec function VoteMap(optional string s)
{
	if ( s == "" )
	{
		ClientMessage("[Server] Type 'VoteMap' followed by part of the map name to vote for a map.");
		ClientMessage("[Server] For example, type 'VoteMap expo' to vote for Import_Export.");
	}
	else
	ClientWantsMapVote(s);
}

//takes a vote and sends to launcher
function ClientWantsMapVote(string s)
{
	if ( GameReplicationInfo.m_eCurrectServerState == GameReplicationInfo.RSS_EndOfMatch )
	{
		ClientMessage("[Error] Match is over. Vote could not be counted.");
		return;
	}
	if ( Launcher != none )
	Launcher.ConvertVote(self,s);
}

//receives a counted ballot from launcher
//1 is next map, 2 is skip map
function Voted(int i)
{
	if ( i == iVoted )//vote is same as last time
	ClientMessage("[Error] You have already voted for this map.");
	else if ( iVoted > 0 )//have already voted but changing vote
	Launcher.ClientVote(self,i,true);
	else//have not yet voted
	Launcher.ClientVote(self,i);
	iVoted = i;
}

//admin can disable map voting
exec function ToggleMapVote()
{
	ClientWantsToggleMapVote();
}

//tell launcher to disable or enable map voting
function ClientWantsToggleMapVote()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	{
		Launcher.SetMapVote(self);
	}
}

//END MAP VOTING

//PAUSES

exec function SetAutoPause(int i)
{
	ClientWantsSetAutoPause(i);
}

//how long should we pause at start of map?
//tells launcher
function ClientWantsSetAutoPause(int i)
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	{
		Launcher.PreMatchPauseTime = i;
		Launcher.SaveConfig();
		if ( i == 0 )
		ClientMessage("[Server] Pre-match auto-pause disabled.");
		else
		ClientMessage("[Server] Pre-match auto-pause set to " $ i $ " seconds.");
	}
}

//admin can pause countdown
exec function Pause()
{
	ClientWantsPause();
}

//got to test, could be bugs if some other player has paused the game
function ClientWantsPause()
{
	if ( !CheckAuthority(1) && !bIsLimitedAdmin )//limited admins can do this
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( !R6GameReplicationInfo(GameReplicationInfo).m_bRepMenuCountDownTimePaused )//not paused, lets pause it
	{
		R6MultiPlayerGameInfo(Level.Game).PauseCountDown();
	}
	else//need to unpause
	{
		R6MultiPlayerGameInfo(Level.Game).UnPauseCountDown();
	}
}

//end pausing

//auto switching on win
exec function ToggleAutoSwitch()
{
	ClientWantsToggleAutoSwitch();
}

//tell launcher if we auto switch maps
function ClientWantsToggleAutoSwitch()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	{
		Launcher.ChangeAutoSwitch(self);
	}
}

//limited admin password
//allows two passwords: main admin one, and limited one
function ServerAdminLogin(string _Password)
{
	if ( ( _Password == LimitedAdminPass ) && ( LimitedAdminPass != "" ) )
    {
		bIsLimitedAdmin = true;
		ClientMessage("[Server] You have successfully logged in as a limited admin on this server.");
		ClientMessage("[Server] You are permitted to kick or switch players, pause the countdown, or change maps.");
		ClientMessage("[Server] Your limited admin authority will be reset at the end of the match.");
		ClientMessage("[Server] Re-login each match in order to use the limited admin powers.");
		return;
	}
	super.ServerAdminLogin(_Password);
}

//set a new limited admin password
exec function NewLimitedPassword(optional string s)
{
	ClientWantsNewLimitedPassword(s);
}

exec function NewLPassword(optional string s)
{
	ClientWantsNewLimitedPassword(s);
}

//tell launcher new limited password
function ClientWantsNewLimitedPassword(optional string s)
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	Launcher.SetNewLimitedPassword(s);
}

//limited admins can change map
function ServerMap(int iGotoMapId,string explanation)
{
    local R6GameReplicationInfo _GRI;
    local R6PlayerController _playerController;
    local string _mapName;
    local string _PlayerName;
	_GRI = R6GameReplicationInfo(GameReplicationInfo);
	if ( bIsLimitedAdmin )//limited admin allowed, otherwise send to super to handle
	{
		if ( iGotoMapId >= _GRI.m_MapLength )
		{
			ClientNoAuthority();
			return;
		}
		_mapName = _GRI.m_mapArray[iGotoMapId];
		_PlayerName = PlayerReplicationInfo.PlayerName;
		foreach AllActors(class'R6PlayerController',_playerController )
		{
			_playerController.ClientServerMap(_PlayerName,_mapName,explanation);
		}
		R6AbstractGameInfo(Level.Game).EndGameAndJumpToMapID(iGotoMapId);
	}
	else
	super.ServerMap(iGotoMapID,explanation);
}

//limited admins can kick
simulated function ProcessKickRequest(R6PlayerController _PlayerController,optional bool bBan)
{
	local string _AdminName,_KickeeName;
	local R6PlayerController _pcIterator;
	if ( bIsLimitedAdmin && !bBan )//limited admins can only kick
	{
		if ( _PlayerController == none )
		{
			ClientKickBadId();
			return;
		}
		if ( ( Viewport(_PlayerController.Player) != none ) || ( _PlayerController.CheckAuthority(Authority_Admin) ) )
		{
			ClientNoKickAdmin();
			return;
		}
		_AdminName = PlayerReplicationInfo.PlayerName;
		_KickeeName = _PlayerController.PlayerReplicationInfo.PlayerName;
		foreach AllActors(class'R6PlayerController', _pcIterator )
		{
			_pcIterator.ClientAdminKickOff(_AdminName,_KickeeName);
		}
		_PlayerController.ClientKickedOut();
		_PlayerController.SpecialDestroy();
	}
	else
	super.ProcessKickRequest(_PlayerController,bBan);//if not a limited admin or not just kick, send to super
}

//---------------------------------------------------------------------------
//								SM2 REPLACEMENT FEATURES
//---------------------------------------------------------------------------

//DEATH MESSAGES

//toggle headshot death messages
exec function ToggleHeadshotMsg()
{
	ClientWantsToggleHeadshotMsg();
}

function ClientWantsToggleHeadshotMsg()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	{
		Launcher.ChangeHeadshots(self);
	}
}

//toggle weapon names on and off
exec function ToggleWeaponName()
{
	ClientWantsWeaponName();
}

function ClientWantsWeaponName()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	{
		Launcher.ChangeWeaponName(self);
	}
}

//returns a randomly chosen death message depending on type sent
function string GetAMessage(int type)
{
	local array<int> ai;
	local int i;
	i = 0;
	if ( Level.NetMode == NM_DedicatedServer )//only on client
	return "";
	switch ( type )
	{
		case 1:
		while ( i < 30 )
		{
			if ( DeathMessage[i] != "" )//there's a death message here
			ai[ai.length] = i;//store it in temp array
			i++;
		}
		return DeathMessage[ai[rand(ai.length)]];//return a random msg
		break;
		case 2:
		while ( i < 30 )
		{
			if ( SuicideMessage[i] != "" )//there's a death message here
			ai[ai.length] = i;//store it in temp array
			i++;
		}
		return SuicideMessage[ai[rand(ai.length)]];//return a random msg
		break;
		case 3:
		while ( i < 30 )
		{
			if ( BombMessage[i] != "" )//there's a death message here
			ai[ai.length] = i;//store it in temp array
			i++;
		}
		return BombMessage[ai[rand(ai.length)]];//return a random msg
		break;
		default:
		while ( i < 30 )
		{
			if ( HeadshotMessage[i] != "" )//there's a death message here
			ai[ai.length] = i;//store it in temp array
			i++;
		}
		return HeadshotMessage[ai[rand(ai.length)]];//return a random msg
		break;
	}
}

//cancel this function so that it doesn't get called
//pawn class will call xClientDeathMessage instead, with weapon name and headshot included
function ClientDeathMessage(string Killer,string Killed,byte bSuicideType);

function xClientDeathMessage(string Killer,string Killed,byte bSuicideType,string sWeapon,bool bHead)
{
	local string sdMsg;
	if ( Level.NetMode == NM_Standalone )
	return;
	if ( myHUD != none )
	{
		if ( ( bSuicideType == DEATHMSG_CONNECTIONLOST ) || ( bSuicideType == DEATHMSG_PENALTY ) || ( bSuicideType == DEATHMSG_SWITCHTEAM ) 
			|| ( bSuicideType == DEATHMSG_HOSTAGE_DIED ) || ( bSuicideType == DEATHMSG_HOSTAGE_KILLEDBY ) || ( bSuicideType == DEATHMSG_HOSTAGE_KILLEDBYTERRO ) )
		super.ClientDeathMessage(Killer,Killed,bSuicideType);//use super version
		else if ( bSuicideType == DEATHMSG_KILLED_BY_BOMB )//BombMessage
		sdMsg = GetAMessage(3);
		else if ( ( bSuicideType == DEATHMSG_KAMAKAZE ) || ( Killer == Killed ) )//SuicideMessage
		{
			sdMsg = GetAMessage(2);
			if ( IncludeWeaponName && ( sWeapon == "explosion" ) )//suicide by explosion
			sdMsg = sdMsg $ " with an " $ sWeapon;
		}
		else//DeathMessage
		{
			if ( bSuicideType == DEATHMSG_RAINBOW_KILLEDBYTERRO )
			Killer = "a terrorist";
			if ( bHead && CallHeadshots )//HeadshotMessage
			sdMsg = GetAMessage(4);
			else
			sdMsg = GetAMessage(1);
			if ( IncludeWeaponName && ( sWeapon != "" ) )
			{
				//check for vowel
				if ( ( caps(Left(sWeapon,1)) == "A" ) || ( caps(Left(sWeapon,1)) == "E" ) || ( caps(Left(sWeapon,1)) == "I" ) || ( caps(Left(sWeapon,1)) == "O" ) || ( caps(Left(sWeapon,1)) == "U" ) )
				sdMsg = sdMsg $ " with an " $ sWeapon;
				else//no vowel
				sdMsg = sdMsg $ " with a " $ sWeapon;
			}
		}
		//replace % with killer, & with victim
		ReplaceText(sdMsg,"%",Killer);
		ReplaceText(sdMsg,"&",Killed);
		//if "a terrorist" at beginning, need capital letter
		if ( caps(left(sdMsg,6)) == "A TERR" )
		ReplaceText(sdMsg,"a terrorist","A terrorist");
		myHUD.AddDeathTextMessage(sdMsg,class'LocalMessage');
	}
}

//these functions will decide whether the client wants to view messages, remove messages, or add messages
//and call the appropriate function
exec function DeathMsg(string s)
{
	if ( ( Caps(Left(s,7)) == "REMOVE " ) || ( Caps(Left(s,4)) == "ADD " ) )
	ClientWantsMessage(1,s);
	else
	DisplayDeathMessages(1);
}

exec function SuicideMsg(string s)
{
	if ( ( Caps(Left(s,7)) == "REMOVE " ) || ( Caps(Left(s,4)) == "ADD " ) )
	ClientWantsMessage(2,s);
	else
	DisplayDeathMessages(2);
}

exec function BombMsg(string s)
{
	if ( ( Caps(Left(s,7)) == "REMOVE " ) || ( Caps(Left(s,4)) == "ADD " ) )
	ClientWantsMessage(3,s);
	else
	DisplayDeathMessages(3);
}

exec function HeadshotMsg(string s)
{
	if ( ( Caps(Left(s,7)) == "REMOVE " ) || ( Caps(Left(s,4)) == "ADD " ) )
	ClientWantsMessage(4,s);
	else
	DisplayDeathMessages(4);
}

//a client-side function
//shows what death messages the client has had sent to them by the server
simulated function DisplayDeathMessages(int type)
{
	local int i;
	local string s;
	local color c;
	c.R = 0;
	c.B = 255;
	c.G = 255;
	i = 0;
	switch ( type )
	{
		case 1://death
		AddMessageToConsole(" ",c);
		AddMessageToConsole(" -- Death Messages --",c);
		while ( i < 30 )
		{
			if ( DeathMessage[i] != "" )
			{
				s = (i + 1) $ ": " $ DeathMessage[i];
				ReplaceText(s,"&","VICTIM");
				ReplaceText(s,"%","KILLER");
				AddMessageToConsole(s,c);
			}
			i++;
		}
		break;
		case 2://suicide
		ClientMessage(" -- Suicide Messages --");
		while ( i < 30 )
		{
			if ( SuicideMessage[i] != "" )
			{
				s = (i + 1) $ ": " $ SuicideMessage[i];
				ReplaceText(s,"&","VICTIM");
				ReplaceText(s,"%","KILLER");
				AddMessageToConsole(s,c);
			}
			i++;
		}
		break;
		case 3://bomb
		ClientMessage(" -- Bomb Messages --");
		while ( i < 30 )
		{
			if ( BombMessage[i] != "" )
			{
				s = (i + 1) $ ": " $ BombMessage[i];
				ReplaceText(s,"&","VICTIM");
				ReplaceText(s,"%","KILLER");
				AddMessageToConsole(s,c);
			}
			i++;
		}
		break;
		default://headshot
		ClientMessage(" -- Headshot Messages --");
		while ( i < 30 )
		{
			if ( HeadshotMessage[i] != "" )
			{
				s = (i + 1) $ ": " $ HeadshotMessage[i];
				ReplaceText(s,"&","VICTIM");
				ReplaceText(s,"%","KILLER");
				AddMessageToConsole(s,c);
			}
			i++;
		}
		break;
	}
	AddMessageToConsole(" ",c);
	AddMessageToConsole("To add a message to the list, make sure you include 'ADD' then the message.",c);
	AddMessageToConsole("You can use 'KILLER' or the '%' symbold in place of the killer's name.",c);
	AddMessageToConsole("You can use 'VICTIM' or the '&' symbold in place of the victim's name.",c);
	AddMessageToConsole("For more details on editing the death messages, use the 'help' command.",c);
}

//called on server by client
//we need to:
// - parse string to see if it's an addition or removal
// - send the string to launcher class
// - launcher class will send vars again to each client to update them and also save config
function ClientWantsMessage(int type,string msg)
{
	local int j;
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Caps(Left(msg,7)) == "REMOVE " )//remove a number
	{
		msg = Mid(msg,7);//get rid of remove text, leave us with number
		j = int(msg);
		if ( j != 0 )//the leftovers are an int
		{
			if ( Launcher != none )
			Launcher.RemoveMsg(type,j,self);
		}
		else
		{
			ClientMessage("[Error] '" $ msg $ "' is not a valid number.");
			return;
		}
	}
	else if ( Caps(Left(msg,4)) == "ADD " )//add a msg
	{
		msg = Mid(msg,4);//get rid of add text, leave us with message
		ReplaceText(msg,"VICTIM","&");//if typed in as victim or killer, change to regular symbol
		ReplaceText(msg,"KILLER","%");
		if ( Launcher != none )
		Launcher.AddMsg(type,msg,self);
	}
}

//end messages

//enable or disable zulu go codes
exec function ToggleZulu()
{
	ClientWantsToggleZulu();
}

function ClientWantsToggleZulu()
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( Launcher != none )
	Launcher.ToggleZulu(self);
}

//switch player to another team
exec function Switch(string s)
{
	ClientWantsSwitch(s);
}

//switches a player
//can intelligently auto-determine team to switch to if not specified
function ClientWantsSwitch(coerce string s)//accessed none?
{
	local R6PlayerController B,C;
	local string x;
	if ( !CheckAuthority(1) && !bIsLimitedAdmin )//limited admins can do this
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( s == "" )
	{
		ClientMessage("[Error] No player chosen. Use 'switch <ID#> <team>' to choose a player to switch teams.");
		return;
	}
	if ( InStr(s," ") == -1 )//only the number was present, no team
	{
		x = s;
		s = "NT";//signify for later that we need to determine team
	}
	else//number and team present
	{
		x = mid(s,0,InStr(s," "));
		s = Caps(Mid(s,len(x)+1));
	}
	B = R6PlayerController(FindPlayer(x,true));
	if ( B == none )
	{
		ClientMessage("[Error] Player #" $ x $ " not found.");
		return;
	}
	if ( ( ( B.m_pawn != none ) && ( B.m_pawn.IsAlive() ) ) 
		|| ( B.GameReplicationInfo.m_bInPostBetweenRoundTime ) 
		|| ( B.GameReplicationInfo.m_bGameOverRep ) )
	{
		ClientMessage("[Error] Cannot switch player at this time.");
		return;
	}
	if ( s == "NT" )//no team specified
	{
		switch ( B.PlayerReplicationInfo.TeamID )
		{
			case int(ePlayerTeamSelection.PTS_Alpha)://green players switch red - or players switch to S
			if ( Level.IsGameTypeTeamAdversarial(Level.Game.m_szGameTypeFlag) )//two teams - can switch to red team
			B.ServerTeamRequested(PTS_Bravo);
			else//only one team - must switch to spectator
			B.ServerTeamRequested(PTS_Spectator);
			break;
			default://red players, spectators switch green
			B.ServerTeamRequested(PTS_Alpha);
			break;
		}
	}
	else if ( left(s,1) == "G" )//green chosen
	B.ServerTeamRequested(PTS_Alpha);
	else if ( ( left(s,1) == "R" ) && ( Level.IsGameTypeTeamAdversarial(Level.Game.m_szGameTypeFlag) ) )//red chosen in a teams game
	B.ServerTeamRequested(PTS_Bravo);
	else//spectator chosen
	B.ServerTeamRequested(PTS_Spectator);
	foreach DynamicActors(class'R6PlayerController',C)
	{
		if ( C != none )
		C.ClientMessage("[Admin] " $ PlayerReplicationInfo.PlayerName $ " has switched " $ B.PlayerReplicationInfo.PlayerName $ "'s team.");
	}
}

//send a large server message
exec function ServerMessage(string s)
{
	ClientWantsBigMsg(s);
}

exec function ServerSay(string s)
{
	ClientWantsBigMsg(s);
}

exec function BigSay(string s)
{
	ClientWantsBigMsg(s);
}

exec function ServerMsg(string s)
{
	ClientWantsBigMsg(s);
}

function ClientWantsBigMsg(string s)
{
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( bIAmMuted )//muted player
	{
		SetMuted(true);
		return;
	}
	else
	R6AbstractGameInfo(Level.Game).BroadcastGameMsg("",s,"");
}

//set all players ready
exec function Start()
{
	ClientWantsStart();
}

function ClientWantsStart()
{
	local R6PlayerController B;
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( ( GameReplicationInfo.m_eCurrectServerState == GameReplicationInfo.RSS_InGameState ) || ( GameReplicationInfo.m_bInPostBetweenRoundTime ) || ( GameReplicationInfo.m_bGameOverRep ) )
	{
		ClientMessage("[Error] Round already started.");
		return;
	}
	foreach DynamicActors(class'R6PlayerController',B)
	{
		B.PlayerReplicationInfo.m_bPlayerReady = true;
		B.ClientMessage("[Admin] " $ PlayerReplicationInfo.PlayerName $ " sets all players ready.");
	}
}

//override parent function
//if you are an admin, you can also see UBI ids
exec function PlayerList()
{
	local PlayerReplicationInfo P;
	local string szId,szName;
	if ( !CheckAuthority(1) )//non admins get regular player list
	{
		super.PlayerList();
		return;
	}
	szId = Localize("Game","Id","R6GameInfo");
	szName = Localize("Game","Name","R6GameInfo");
	foreach AllActors(class'PlayerReplicationInfo',P)
	{
		AddMessageToConsole(szId $ ": " $ P.PlayerID $ "   " $ szName $ ": " $ P.PlayerName $ "   UBI: " $ P.m_szUbiUserID,myHUD.m_ServerMessagesColor);
	}
}

exec function Chat(string s)
{
	ClientWantsChat(s);
}

//server function
//sends chat to chosen client
function ClientWantsChat(string s)
{
	local string n;
	local R6PlayerController B;
	if ( bIAmMuted )//muted player
	{
		SetMuted(true);
		return;
	}
	if ( s == "" )
	{
		ClientMessage("[Error] No player chosen. Use 'chat <ID#>' to choose a player to chat with.");
		return;
	}
	n = Mid(s,0,InStr(s," "));
	ReplaceText(s,n $ " ","");
	B = R6PlayerController(FindPlayer(n,true));
	if ( B != none )
	{
		ClientMessage("[Private chat with " $ B.PlayerReplicationInfo.PlayerName $ "] " $ s);
		B.ClientMessage(PlayerReplicationInfo.PlayerName $ " [Private chat]: " $ s);
		//BroadcastText(PlayerReplicationInfo,B,PlayerReplicationInfo.PlayerName $ " [Private chat]: " $ s,'ServerMessage');
	}
	else
	ClientMessage("[Error] Player #" $ n $ " not found");
	return;
}

exec function Gimp(string s)
{
	ClientWantsGimp(s);
}

//server function
//reliable when called from client
//injures a player
function ClientWantsGimp(coerce string s)
{
	local R6PlayerController B,C;
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( s == "" )
	{
		ClientMessage("[Error] No player chosen. Use 'gimp <ID#>' to choose a player to gimp.");
		return;
	}
	B = R6PlayerController(FindPlayer(s,true));
	if ( B != none )
	{
		if ( ( B.m_pawn == none ) || ( !B.m_pawn.IsAlive() ) || ( B.GameReplicationInfo.m_eCurrectServerState != B.GameReplicationInfo.RSS_InGameState ) || ( B.GameReplicationInfo.m_bInPostBetweenRoundTime ) || ( B.GameReplicationInfo.m_bGameOverRep ) )
		ClientMessage("[Error] Player could not be gimped.");
		else
		{
			B.m_pawn.ServerForceKillResult(2);
			B.m_pawn.R6TakeDamage(999999,999999,none,B.m_pawn.Location,B.m_pawn.Location,0);
			B.m_pawn.ServerForceKillResult(0);
			foreach DynamicActors(class'R6PlayerController',C)
			{
				if ( C != none )
				C.ClientMessage("[Admin] " $ PlayerReplicationInfo.PlayerName $ " hurts " $ B.PlayerReplicationInfo.PlayerName $ " as punishment.");
			}
		}
	}
	else
	ClientMessage("[Error] Player #" $ s $ " not found.");
	return;
}

exec function Kill(string s)
{
	ClientWantsKill(s);
}

//server function
//kills chosen client
function ClientWantsKill(coerce string s)
{
	local R6PlayerController B,C;
	if ( !CheckAuthority(1) )
	{
		ClientMessage("[Error] You do not have the authority to execute that command.");
		return;
	}
	if ( s == "" )
	{
		ClientMessage("[Error] No player chosen. Use 'kill <ID#>' to choose a player to kill.");
		return;
	}
	B = R6PlayerController(FindPlayer(s,true));
	if ( B != none )
	{
		if ( ( !B.m_pawn.IsAlive() ) || ( B.GameReplicationInfo.m_eCurrectServerState != B.GameReplicationInfo.RSS_InGameState ) || ( B.GameReplicationInfo.m_bInPostBetweenRoundTime ) || ( B.GameReplicationInfo.m_bGameOverRep ) )
		ClientMessage("[Error] Player could not be killed.");
		else
		{
			R6Pawn(B.Pawn).ServerSuicidePawn(2);
			foreach DynamicActors(class'R6PlayerController',C)
			{
				C.ClientMessage("[Admin] " $ PlayerReplicationInfo.PlayerName $ " kills " $ B.PlayerReplicationInfo.PlayerName $ " as punishment.");
			}
		}
	}
	else
	ClientMessage("[Error] Player #" $ s $ " not found.");
}

//---------------------------------------------------------------------------
//								END SM2
//---------------------------------------------------------------------------

defaultproperties
{
	fAim=1.0//default is normal accuracy
	helpfiles(0)="The Ultimate Server Mod was designed by .Twi as the perfect addition to your multiplayer server."
	helpfiles(1)="The Ultimate Server Mod was developed with assistance from ShadowSquad clan and iD.Tango."
	helpfiles(2)="Check out .Twi's other mods, including the R6: Zombies mod for both single- and multi-player."
	helpfiles(3)="Looking for an anticheat mod? Try Sentinel by .Twi."
	helpfiles(4)="'The worst part about the internet is you can make up a quote and attribute it to anyone' - Abraham Lincoln"
	helpfiles(5)="To chat privately, type 'chat' then the player's ID number."
	helpfiles(6)="You can type the command 'playerlist' to see all players' IDs."
	helpfiles(7)="To wound a player as punishment for poor behaviour, use the 'gimp' command."
	helpfiles(8)="Type 'gimp' then the player's ID number to hurt them."
	helpfiles(9)="The Ultimate Server Tools Mod was created by .Twi."
	helpfiles(10)="Ultimate Server Tools Mod is not compatible with Server Mod (SM1 or SM2) or with Run N Gun mods."
	helpfiles(11)="If you need to kill a player who is troublesome, use the 'kill' command."
	helpfiles(12)="Type 'kill' then the player's ID number to kill them."
	helpfiles(13)="The 'playerlist' command now shows each player's UBI ID and whether they are a server admin."
	helpfiles(14)="For non-admin players, the 'playerlist' command works like it always did."
	helpfiles(15)="If you are tired of waiting between rounds, type 'start' into the console to start the round instantly."
	helpfiles(16)="To send a big server message to everyone, use 'serversay' then your message."
	helpfiles(17)="The commands 'servermsg' and 'servermessage' do the same thing as 'serversay'."
	helpfiles(18)="To switch a player's team, use the 'switch' command, followed by the player's ID number."
	helpfiles(19)="When switching a player, if you do not specify a team color the mod will automatically put them on the opposite team."
	helpfiles(20)="When choosing a team color to switch a player to, you can use 'G' instead of 'green', 'S' instead of 'spectator', etc."
	helpfiles(21)="You can use the 'pause' command to stop or start the pre-round countdown."
	helpfiles(22)="If you want to have the server automatically pause at the beginning of a new match, use 'setautopause' then the number of seconds."
	helpfiles(23)="To disable auto-pausing, try 'setautopause 0'."
	helpfiles(24)="To view the server's death messages, type 'deathmsg'."
	helpfiles(25)="To view the server's suicide messages, type 'suicidemsg'."
	helpfiles(26)="To view the server's bomb messages, type 'bombmsg'."
	helpfiles(27)="To view the server's headshot messages, type 'headshotmsg'."
	helpfiles(28)="To toggle whether or not the death messages include the weapon name, try the command 'toggleweaponname'."
	helpfiles(29)="To remove a death message from the list, try 'deathmsg REMOVE <#>'."
	helpfiles(30)="To remove a suicide message from the list, try 'suicidemsg REMOVE <#>'."
	helpfiles(31)="To remove a bomb message from the list, try 'bombmsg REMOVE <#>'."
	helpfiles(32)="To remove a headshot message from the list, try 'headshotmsg REMOVE <#>'."
	helpfiles(33)="When working with death messages, use '&' for the victim's name and '%' for the killer's name."
	helpfiles(34)="When working with suicide messages, use '&' for the victim's name."
	helpfiles(35)="When working with bomb messages, use '&' for the victim's name."
	helpfiles(36)="When working with headshot messages, use '&' for the victim's name and '%' for the killer's name."
	helpfiles(37)="To add a new death message, try 'deathmsg ADD <new message>'."
	helpfiles(38)="To add a new suicide message, try 'suicidemsg ADD <new message>'."
	helpfiles(39)="To add a new bomb message, try 'bombmsg ADD <new message>'."
	helpfiles(40)="To add a new headshot message, try 'headshotmsg ADD <new message>'."
	helpfiles(41)="You can enable or disable the use of headshot messages on the server using the 'toggleheadshotmsg' command."
	helpfiles(42)="To mute an annoying player, try the 'mute' command, followed by the player's ID number."
	helpfiles(43)="If a player is already muted, the 'mute' command followed by their ID number will unmute them."
	helpfiles(44)="If you have been muted, and would like to speak, you can use the 'askunmute' command."
	helpfiles(45)="The 'askunmute' command will ask any admins on the server to unmute you."
	helpfiles(46)="If zulu spam is a problem, try the command 'togglezulu'."
	helpfiles(47)="The 'togglezulu' command will completely disable or enable the use of the zulu go-code."
	helpfiles(48)="The 'toggleautoswitch' command will turn on or off map auto-switching based on rounds won."
	helpfiles(49)="If 'toggleautoswitch' is on, the map will change as soon as a player or team wins at least half the total rounds."
	helpfiles(50)="You can get easy access to most mod settings using the Settings App."
	helpfiles(51)="Type 'settings' in the console to open the Settings App window."
	helpfiles(52)="The Settings App can be moved with the mouse, and stays on-screen until closed."
	helpfiles(53)="Easily manage players using the Players App."
	helpfiles(54)="Type 'players' in the console to open the Players App window."
	helpfiles(55)="The Players App can be moved with the mouse, and stays on-screen until closed."
	helpfiles(56)="In the Players App, use the 'refresh' button if the amount of players in the server changes."
	helpfiles(57)="Feel like playing something besides Ravenshield? Bored between rounds? Try the console command 'distract' for a new game."
	helpfiles(58)="You can use 'newlimitedpassword' followed by a password to give people limited admin powers."
	helpfiles(59)="Logging in with the limited admin password will allow you to kick or switch players, pause countdown, or change maps."
	helpfiles(60)="Changing the limited admin password will disable limited admin powers on anyone logged in with the old pass."
	helpfiles(61)="You can disable limited admin powers completely by setting the password to nothing."
	helpfiles(62)="Limited admin powers only last for a single match. Limited admins will need to re-login in a later match."
	helpfiles(63)="For a list of all new console commands, try typing 'help all'."
	helpfiles(64)="To allow or disallow players to vote for what map they want next, try 'togglemapvote'."
	helpfiles(65)="If enabled, players can vote for what map they want next with the command 'votemap'."
	helpfiles(66)="Type 'votemap' followed by part of the map name to cast your vote for the map to play next."
	helpfiles(67)="Type 'setaccuracy' followed by a number to modify all weapons' accuracy."
	helpfiles(68)="Default weapon accuracy for the 'setaccuracy' command is 1.0."
	helpfiles(69)="To get accuracy values similar to the Run N Gun mod, try 'setaccuracy 0.5'."
	helpfiles(70)="With the 'setaccuracy' command, numbers smaller than 1.0 will make weapons more accurate. Larger than 1.0 is less accurate."
	helpfiles(71)="If you want to prevent people from stealing a player's name, try the 'register' command."
	helpfiles(72)="Type 'register' followed by the player's number and the name you want to register to ensure only the player can use that name."
	helpfiles(73)="You can use the 'unregister' command followed by a name to allow anyone to use that name again."
	helpfiles(74)="You can register a player's name through the Players App, in addition to using the 'register' command."
	helpfiles(75)="To view the list of names that are registered, type the command 'listnames' (without anything else) into the console."
	helpfiles(76)="To learn about some hidden features and easter eggs of the mod, try the 'distract' console command."
	helpfiles(77)="The 'register' command can be used to prevent players from using crude words in names. Just register the word to yourself."
	helpfiles(78)="If your server often fills up with spectators, try the 'togglekickspectators' command."
	helpfiles(79)="The 'togglekickspectators' command will tell the server to automatically kick a random spectator if the server is full."
}